mod coloration;
mod species;
mod stats;

use coloration::Coloration;
use species::Parts;
use stats::Stats;

use crate::genetics::{Genetic, Genotype, Transcriber};

#[derive(Clone, Debug)]
pub struct Creature {
    // pub name: String,
    pub genotype: Genotype,
}

impl Creature {
    pub fn profile(&self) -> String {
        let phenotype: Phenotype = self.genotype.load();

        let mut profile = phenotype.profile(); // format!("Name: {}\n{}", self.name, phenotype.profile());
        profile.push_str("\nGenome:\n");
        for (chromosome, (lhs, rhs)) in self.genotype.chromosomes.iter() {
            profile.push_str(&format!("  {}[l]: {}\n", chromosome, lhs));
            profile.push_str(&format!("  {}[r]: {}\n", chromosome, rhs));
        }
        return profile;
    }

    pub fn breed(&self, other: &Creature) -> Creature {
        return Creature {
            // name: String::from("Pyrex"),
            genotype: self.genotype.breed(&other.genotype),
        };
    }
}

#[derive(Default)]
pub struct Phenotype {
    pub stats: Stats,
    pub coloration: Coloration,
    pub parts: Parts,
}

impl Phenotype {
    pub fn profile(&self) -> String {
        format!(
            "{}{}{}",
            self.stats.profile(),
            self.coloration.profile(),
            self.parts.profile()
        )
    }
}

impl Genetic for Phenotype {
    fn transcribe(&mut self, t: Transcriber) {
        self.stats.transcribe(t.nest("stats"));
        self.coloration.transcribe(t.nest("coloration"));
        self.parts.transcribe(t.nest("parts"));
    }
}
