use choice_function::ChoiceFunction;
use creature::{Creature, Phenotype};
use genetics::Genotype;
use rand::{rng, seq::SliceRandom};

mod choice_function;
mod creature;
mod genetics;

fn generate_creature() -> Creature {
    let genotype: Genotype = Genotype::generate::<Phenotype>(ChoiceFunction::Random);
    let creature = Creature {
        // name: String::from("Pyrex"),
        genotype,
    };
    return creature;
}

fn evolve() {
    const N_ITERATIONS: usize = 200;
    const N_CREATURES: usize = 100;
    const N_SURVIVORS: usize = 90;

    let mut pool = vec![];
    for _ in 0..N_CREATURES {
        pool.push(generate_creature());
    }

    for iteration in 0..N_ITERATIONS {
        report(iteration, &pool);

        // kill N creatures
        let mut survivors = pool.clone();
        survivors.sort_by(|x, y| fitness(x).partial_cmp(&fitness(y)).unwrap());
        survivors.reverse();
        survivors.drain(N_SURVIVORS..N_CREATURES);

        let mut pool2 = vec![];
        let mut survivors_queue = vec![];
        while survivors_queue.len() < N_CREATURES * 2 {
            survivors_queue.extend(survivors.clone());
        }
        survivors_queue.shuffle(&mut rng());

        for _ in 0..N_CREATURES {
            let lhs = survivors_queue.pop().unwrap();
            let rhs = survivors_queue.pop().unwrap();
            pool2.push(lhs.breed(&rhs));
        }
        pool = pool2;
    }
}

fn report(iteration: usize, pool: &[Creature]) {
    println!("== Iteration {} ==", iteration + 1);
    println!("{}", pool[0].profile());
    println!("");
}

fn fitness(creature: &Creature) -> f64 {
    let phenotype: Phenotype = creature.genotype.load();
    let mut value = 0.0;
    if phenotype.parts.has_wings() {
        value += 0.5;
    }
    value -= phenotype.coloration.body.r.value() as f64 / 4.0;
    value += (phenotype.stats.iq.value() as f64) / 10.0;

    return value;
}

fn main() {
    evolve();
}
